@@ -1,14 +1,8 @@
-{-# LANGUAGE BangPatterns        #-}
-{-# LANGUAGE CPP                 #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
-{-# LANGUAGE MagicHash           #-}
-{-# LANGUAGE Rank2Types          #-}
-{-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TypeFamilies        #-}
-{-# LANGUAGE FlexibleContexts    #-}
+{-# LANGUAGE BangPatterns, CPP, DeriveDataTypeable, FlexibleContexts,
+    MagicHash, Rank2Types, ScopedTypeVariables, TypeFamilies, UnboxedTuples #-}
 -- |
 -- Module    : System.Random.MWC
--- Copyright : (c) 2009, 2010 Bryan O'Sullivan
+-- Copyright : (c) 2009, 2010, 2011 Bryan O'Sullivan
 -- License   : BSD3
 --
 -- Maintainer  : bos@serpentine.com
@@ -30,6 +24,8 @@ module System.Random.MWC
     , GenIO
     , GenST
     , Seed
+    , fromSeed
+    , toSeed
     , Variate(..)
     -- * Other distributions
     , normal
@@ -54,7 +50,7 @@ import Control.Exception       (IOException, catch)
 import Control.Monad           (ap, liftM, unless)
 import Control.Monad.Primitive (PrimMonad, PrimState, unsafePrimToIO)
 import Control.Monad.ST        (ST)
-import Data.Bits               ((.&.), (.|.), xor)
+import Data.Bits               (Bits, (.&.), (.|.), shiftL, shiftR, xor)
 import Data.Int                (Int8, Int16, Int32, Int64)
 import Data.IORef              (atomicModifyIORef, newIORef)
 import Data.Ratio              ((%), numerator)
@@ -64,8 +60,6 @@ import Data.Vector.Generic     (Vector, unsafeFreeze)
 import Data.Word               (Word, Word8, Word16, Word32, Word64)
 import Foreign.Marshal.Alloc   (allocaBytes)
 import Foreign.Marshal.Array   (peekArray)
-import GHC.Base       (Int(I#))
-import GHC.Word       (Word64(W64#), uncheckedShiftL64#, uncheckedShiftRL64#)
 import Prelude hiding (catch)
 import qualified Data.Vector.Generic         as G
 import qualified Data.Vector.Generic.Mutable as GM
@@ -102,9 +96,6 @@ class M.Unbox a => Variate a where
     --   statistical calculations that require non-zero values
     --   (e.g. uses of the 'log' function).
     --
-    -- * The range of random 'Integer' variates is the same as for
-    --   'Int'.
-    --
     -- To generate a 'Float' variate with a range of [0,1), subtract
     -- 2**(-33).  To do the same with 'Double' variates, subtract
     -- 2**(-53).
@@ -237,9 +228,9 @@ instance (Variate a, Variate b, Variate c, Variate d) => Variate (a,b,c,d) where
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
-wordsTo64Bit :: Integral a => Word32 -> Word32 -> a
+wordsTo64Bit :: (Integral a) => Word32 -> Word32 -> a
 wordsTo64Bit x y =
-    fromIntegral ((fromIntegral x `shiftL` 32) .|. fromIntegral y)
+    fromIntegral ((fromIntegral x `shiftL` 32) .|. fromIntegral y :: Word64)
 {-# INLINE wordsTo64Bit #-}
 
 wordToBool :: Word32 -> Bool
@@ -288,35 +279,57 @@ create = initialize defaultSeed
 --
 -- Examples:
 --
--- > initialize (singletonU 42)
+-- > initialize (singleton 42)
 --
--- > initialize (toU [4, 8, 15, 16, 23, 42])
+-- > initialize (toList [4, 8, 15, 16, 23, 42])
 --
 -- If a seed contains fewer than 256 elements, it is first used
 -- verbatim, then its elements are 'xor'ed against elements of the
 -- default seed until 256 elements are reached.
-initialize :: PrimMonad m => I.Vector Word32 -> m (Gen (PrimState m))
+--
+-- If a seed contains exactly 258 elements last two elements are used
+-- to set generator state. It's to ensure that @gen' == gen@
+--
+-- > gen' <- initialize . fromSeed =<< save
+initialize :: (PrimMonad m, Vector v Word32) =>
+              v Word32 -> m (Gen (PrimState m))
 initialize seed = do
     q <- M.unsafeNew 258
     fill q
-    M.unsafeWrite q ioff 255
-    M.unsafeWrite q coff 362436
+    if fini == 258
+      then do
+        M.unsafeWrite q ioff $ G.unsafeIndex seed ioff .&. 255
+        M.unsafeWrite q coff $ G.unsafeIndex seed coff
+      else do
+        M.unsafeWrite q ioff 255
+        M.unsafeWrite q coff 362436
     return (Gen q)
   where fill q = go 0 where
           go i | i == 256  = return ()
                | otherwise = M.unsafeWrite q i s >> go (i+1)
             where s | i >= fini = if fini == 0
-                                  then I.unsafeIndex defaultSeed i
-                                  else I.unsafeIndex defaultSeed i `xor`
-                                       I.unsafeIndex seed (i `mod` fini)
-                    | otherwise = I.unsafeIndex seed i
-        fini = I.length seed
+                                  then G.unsafeIndex defaultSeed i
+                                  else G.unsafeIndex defaultSeed i `xor`
+                                       G.unsafeIndex seed (i `mod` fini)
+                    | otherwise = G.unsafeIndex seed i
+        fini = G.length seed
 {-# INLINE initialize #-}
                                
 -- | An immutable snapshot of the state of a 'Gen'.
-newtype Seed = Seed (I.Vector Word32)
+newtype Seed = Seed { 
+    -- | Convert seed into vector.
+    fromSeed :: I.Vector Word32 
+    }
     deriving (Eq, Show, Typeable)
 
+-- | Convert vector to 'Seed'. It acts similarily to 'initialize' and
+-- will accept any vector. If you want to pass seed immediately to
+-- restore you better call initialize directly since following law holds:
+--
+-- > restore (toSeed v) = initialize v
+toSeed :: (Vector v Word32) => v Word32 -> Seed
+toSeed v = Seed $ I.create $ do { Gen q <- initialize v; return q }
+
 -- Safe version of unsafeFreeze.
 -- NOTE: vector-0.7 will provide function `freeze' with same
 --       functionality. This function shall be removed when support for
@@ -332,6 +345,7 @@ save :: PrimMonad m => Gen (PrimState m) -> m Seed
 save (Gen q) = Seed `liftM` safeFreeze q
 {-# INLINE save #-}
 
+-- NOTE: with vector-0.7 all code could be replaced with `clone'
 -- | Create a new 'Gen' that mirrors the state of a saved 'Seed'.
 restore :: PrimMonad m => Seed -> m (Gen (PrimState m))
 restore (Seed s) = M.unsafeNew n >>= fill
@@ -340,18 +354,27 @@ restore (Seed s) = M.unsafeNew n >>= fill
                 | otherwise = M.unsafeWrite q i (I.unsafeIndex s i) >> go (i+1)
         n = I.length s
 {-# INLINE restore #-}
-  
--- | Using the current time as a seed, perform an action that uses a
--- random variate generator.  This is a horrible fallback for Windows
--- systems.
-withTime :: (PrimMonad m) => (Gen (PrimState m) -> m a) -> IO a
-withTime act = do
+
+
+-- Aquire seed from current time. This is horrible fallback for
+-- Windows system.
+acquireSeedTime :: IO [Word32]
+acquireSeedTime = do
   c <- (numerator . (%cpuTimePrecision)) `liftM` getCPUTime
   t <- toRational `liftM` getPOSIXTime
   let n    = fromIntegral (numerator t) :: Word64
-      seed = [fromIntegral c, fromIntegral n, fromIntegral (n `shiftR` 32)]
-  unsafePrimToIO $ initialize (I.fromList seed) >>= act
+  return [fromIntegral c, fromIntegral n, fromIntegral (n `shiftR` 32)]
 
+-- Aquire seed from /dev/urandom
+acquireSeedSystem :: IO [Word32]
+acquireSeedSystem = do
+  let nbytes = 1024
+      random = "/dev/urandom"
+  allocaBytes nbytes $ \buf -> do
+    nread <- withBinaryFile random ReadMode $
+               \h -> hGetBuf h buf nbytes
+    peekArray (nread `div` 4) buf
+  
 -- | Seed a PRNG with data from the system's fast source of
 -- pseudo-random numbers (\"\/dev\/urandom\" on Unix-like systems),
 -- then run the given action.
@@ -361,40 +384,29 @@ withTime act = do
 -- clock instead). As a result, the sequences it generates may not be
 -- highly independent.
 withSystemRandom :: PrimMonad m => (Gen (PrimState m) -> m a) -> IO a
-withSystemRandom act = tryRandom `catch` \(_::IOException) -> do
+withSystemRandom act = do
+  seed <- acquireSeedSystem `catch` \(_::IOException) -> do
     seen <- atomicModifyIORef warned ((,) True)
     unless seen $ do
-      hPutStrLn stderr ("Warning: Couldn't open " ++ show random)
+      hPutStrLn stderr ("Warning: Couldn't open /dev/urandom")
       hPutStrLn stderr ("Warning: using system clock for seed instead " ++
                         "(quality will be lower)")
-    withTime act
-  where tryRandom = do
-          let nbytes = 1024
-          ws <- allocaBytes nbytes $ \buf -> do
-                  nread <- withBinaryFile random ReadMode $
-                           \h -> hGetBuf h buf nbytes
-                  peekArray (nread `div` 4) buf
-          unsafePrimToIO $ initialize (I.fromList ws) >>= act
-        random = "/dev/urandom"
-        warned = unsafePerformIO $ newIORef False
-        {-# NOINLINE warned #-}
-
--- | Unchecked 64-bit left shift.
-shiftL :: Word64 -> Int -> Word64
-shiftL (W64# x#) (I# i#) = W64# (x# `uncheckedShiftL64#` i#)
-
--- | Unchecked 64-bit right shift.
-shiftR :: Word64 -> Int -> Word64
-shiftR (W64# x#) (I# i#) = W64# (x# `uncheckedShiftRL64#` i#)
+    acquireSeedTime
+  unsafePrimToIO $ initialize (I.fromList seed) >>= act
+  where
+    warned = unsafePerformIO $ newIORef False
+    {-# NOINLINE warned #-}
 
 -- | Compute the next index into the state pool.  This is simply
 -- addition modulo 256.
 nextIndex :: Integral a => a -> Int
 nextIndex i = fromIntegral j
     where j = fromIntegral (i+1) :: Word8
+{-# INLINE nextIndex #-}
 
 a :: Word64
 a = 1540315826
+{-# INLINE a #-}
 
 uniformWord32 :: PrimMonad m => Gen (PrimState m) -> m Word32
 uniformWord32 (Gen q) = do
@@ -404,11 +416,8 @@ uniformWord32 (Gen q) = do
   let t  = a * qi + c
       c' = fromIntegral (t `shiftR` 32)
       x  = fromIntegral t + c'
-      x_lt_c = x < c'
-      x'  | x_lt_c    = x + 1
-          | otherwise = x
-      c'' | x_lt_c    = c' + 1
-          | otherwise = c'
+      (# x', c'' #)  | x < c'    = (# x + 1, c' + 1 #)
+                     | otherwise = (# x,     c' #)
   M.unsafeWrite q i x'
   M.unsafeWrite q ioff (fromIntegral i)
   M.unsafeWrite q coff (fromIntegral c'')
@@ -431,19 +440,13 @@ uniform2 f (Gen q) = do
   let t   = a * qi + c
       c'  = fromIntegral (t `shiftR` 32)
       x   = fromIntegral t + c'
-      x_lt_c = x < c'
-      x'  | x_lt_c    = x + 1
-          | otherwise = x
-      c'' | x_lt_c    = c' + 1
-          | otherwise = c'
+      (# x', c'' #)  | x < c'    = (# x + 1, c' + 1 #)
+                     | otherwise = (# x,     c' #)
       u   = a * qj + fromIntegral c''
       d'  = fromIntegral (u `shiftR` 32)
       y   = fromIntegral u + d'
-      y_lt_d = y < d'
-      y'  | y_lt_d    = y + 1
-          | otherwise = y
-      d'' | y_lt_d    = d' + 1
-          | otherwise = d'
+      (# y', d'' #)  | y < d'    = (# y + 1, d' + 1 #)
+                     | otherwise = (# y,     d' #)
   M.unsafeWrite q i x'
   M.unsafeWrite q j y'
   M.unsafeWrite q ioff (fromIntegral j)
@@ -502,13 +505,9 @@ uniformRange (x1,x2) g
 -- | Generate a vector of pseudo-random variates.  This is not
 -- necessarily faster than invoking 'uniform' repeatedly in a loop,
 -- but it may be more convenient to use in some situations.
-uniformVector :: (PrimMonad m, Variate a)
-             => Gen (PrimState m) -> Int -> m (I.Vector a)
-uniformVector gen n = do
-  mu <- M.unsafeNew n
-  let go !i | i < n     = uniform gen >>= M.unsafeWrite mu i >> go (i+1)
-            | otherwise = unsafeFreeze mu
-  go 0
+uniformVector :: (PrimMonad m, Variate a, Vector v a)
+             => Gen (PrimState m) -> Int -> m (v a)
+uniformVector gen n = G.replicateM n (uniform gen)
 {-# INLINE uniformVector #-}
 
 data T = T {-# UNPACK #-} !Double {-# UNPACK #-} !Double
@@ -549,9 +548,11 @@ normal gen = loop
                            h  = sqrt (-2 * log (v / b + g))
                        in Just (h, u)
         v            = 9.91256303526217e-3
+    {-# NOINLINE blocks #-}
     r                = 3.442619855899
     ratios           = I.zipWith (/) (I.tail blocks) blocks
-    normalTail neg   = tailing
+    {-# NOINLINE ratios #-}
+    normalTail neg  = tailing
       where tailing  = do
               x <- ((/r) . log) `liftM` uniform gen
               y <- log          `liftM` uniform gen
@@ -605,6 +606,7 @@ defaultSeed = I.fromList [
   0x4eb3782d, 0xc3f1e669, 0x61ff8d9e, 0x0909238d, 0xef406165, 0x09c1d762,
   0x705d184f, 0x188f2cc4, 0x9c5aa12a, 0xc7a5d70e, 0xbc78cb1b, 0x1d26ae62,
   0x23f96ae3, 0xd456bf32, 0xe4654f55, 0x31462bd8 ]
+{-# NOINLINE defaultSeed #-}
 
 -- $references
 --
